package com.caimei365.user.service.impl;

import com.caimei365.user.model.JsonModel;
import com.caimei365.user.service.RedisService;
import com.caimei365.user.service.UserService;
import com.caimei365.user.utils.ImageCaptchaUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;


/**
 * Description
 *
 * @author : Charles
 * @date : 2021/3/4
 */
@Slf4j
@Service
public class UserServiceImpl implements UserService {

    @Resource
    private RedisService redisService;

    /**
     * 获取图片验证码
     *
     * @param platformType 0:www，1:crm/h5，2:小程序
     */
    @Override
    public JsonModel<Map<String, Object>> getCaptchaImage(Integer platformType) {
        Map<String, Object> result = new HashMap<>(2);
        try {
            ImageCaptchaUtil.Captcha captcha = null;
            String captchaKey = "";
            while (captchaKey.isEmpty()) {
                captcha = ImageCaptchaUtil.getCaptcha(200, 80, 4);
                captchaKey = ImageCaptchaUtil.getCaptchaKey(captcha.getMd5Code(), platformType);
                boolean exists = redisService.exists(captchaKey);
                // 为了防止重复的图片验证码, 存在就循环重新生成
                if (!exists) {
                    // 保存5分钟
                    redisService.set(captchaKey, captcha.getCode(), 5 * 60L);
                    break;
                }
            }
            result.put("baseImage", captcha.getBase64Image());
            result.put("token", captcha.getMd5Code());
            return JsonModel.success(result);
        } catch (Exception e) {
            return JsonModel.error("获取图片验证码异常", null);
        }
    }
}
